#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int usage(char *msg,char *arg0)
{
 fprintf(stderr,
         "Usage: %s [file.lod] [file.pci]\n"
         "\tReads .lod text format, writes PCI-loadable binary file.\n\n"
         "\tPCI-file text format:\n"
         "\t\t32-bit word 1: Count 32-bit words of program\n"
         "\t\t32-bit word 2: Start address of program\n"
         "\t\t32-bit word 3 and following: Program words\n\n",arg0);
 if(msg) fprintf(stderr,"Error: %s\n",msg);
 return -1;
}

int seekfor(FILE *f,char *string)
{
 int len=strlen(string);
 const char *element="%1023[^\n\r\t ]%*[\n\r\t ]";
 char buffer[1024];
 do
  {
   if(feof(stdin)||ferror(stdin)||(scanf(element,buffer)!=1))
     return 0;
  }
 while(strncmp(buffer,string,len));
 return 1;
}

#define MAX_WORDS 4096
int main(int argc, char **argv)
{
 FILE *in,*out;
 const char *hexnumber="%lx";
 unsigned long i,wordcount;
 unsigned long progstart;
 unsigned long words[MAX_WORDS]; /* Mehr passt auch nicht in den Speicher */
                                 /* auf der DSP-Karte. */
 if(argc>3||argv[1][0]=='-') return usage(NULL,argv[0]);
 in=(argc>1)?freopen(argv[1],"r",stdin):stdin;
 if(!in) return usage("Input file not found.",argv[0]);
 out=(argc>2)?freopen(argv[2],"w",stdout):stdout;
 if(!out) return usage("Output file not found.",argv[0]);
 if(!seekfor(in,"_DATA")) return usage("This is not a .lod input file.",argv[0]);
 if(!seekfor(in,"P")) return usage(".lod file does not contain Program start address.",argv[0]);
 if(fscanf(in,hexnumber,&progstart)!=1) return usage("Invalid program start address.",argv[0]);
 for(wordcount=0;wordcount<MAX_WORDS;wordcount++)
   if(feof(in)||ferror(in)||(fscanf(in,hexnumber,&words[wordcount])!=1))
     break;
 /* PCI-Datei schreiben */
 fprintf(out,"%lx\n",wordcount);
 fprintf(out,"%lx\n",progstart);
 for(i=0;i<wordcount;i++) fprintf(out,"%08lx\n",words[i]);
 if(out!=stdout) fclose(out);
 if(in!=stdin)   fclose(in);
 return 0;
}
