/* Test for Motorola DSP driver and benchmark program. */
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <linux/fs.h>
#include "mdsp.h"

/* Simple DSP program, reads from and writes to HI32 in a loop */
unsigned long prog[]={
0x00000009, /* Length */
0x00000100, /* Address */
0x00200013, /* DSP prog */
0x000cc902,
0x00000003,
0x00084e0b,
0x000cc901,
0x00fffffd,
0x0008ce0d,
0x00014180,
0x000c0101}; 

void success(int yes)
{
 printf(yes?"success.\n":"failed.\n");
}

#define BUFSIZE 0x10000
int main(int argc, char **argv)
{
 const struct timeval tc={10,0};
 struct timeval tv=tc,tv2;
 fd_set fds;
 unsigned long val,oldval,totalbytes,msecs;
 int result;
 int fd;
 int i;
 struct stat fdstat;
 char *name;
 __u32  buffer[BUFSIZE];
 if(argc<2||!strcmp(argv[1],"-h"))
  {
   fprintf(stderr,"Usage: %s /dev/mdsp[1-16]\n",argv[0]);
   return 1;
  }
 name=argv[1];
 if((result=stat(name,&fdstat))!=0)
  {
   fprintf(stderr,"Error: '%s' does not exist, errno=%d\n",name,errno);
   return 2;
  }
 if(MAJOR(fdstat.st_rdev)!=MDSP_MAJOR)
  {
   fprintf(stderr,
      "Error: '%s' is not a  Motorola DSP56xxx device file (major != %d)\n",
      name,MAJOR(fdstat.st_rdev));
   return 3;
  }
 printf("Device file '%s': Major %d, Minor %d.\n",name,
         MAJOR(fdstat.st_rdev),MINOR(fdstat.st_rdev));
 if((fd=open(argv[1],O_RDWR))<0)
  {
   fprintf(stderr,
      "Error: Can't open '%s' in read/write mode.\n",name);
   return 3;
  }
 if(ioctl(fd,MDSPCTL_READ_HSTR,&val))
  {
   fprintf(stderr,"ioctl() failed.");
   close(fd);
   return 4;
  }
 printf("Status register: 0x%lx\n",val);
 ioctl(fd,MDSPCTL_READ_HCTR,&val);
 printf("Control register: 0x%lx\n",val);
 if(val&HCTR_HF0)
  {
   printf("WARNING: HF0 is set, so there will be no upload of test program.\n"
          "Hope the right program is already running!\n");
  }
 else
  {
   printf("Trying to set control register for upload: "); fflush(stdout);
   oldval=val;
   val=((val&~HCTR_HTF1)&~HCTR_HF0)|HCTR_HTF0; /* 3-Byte transfer mode */
   ioctl(fd,MDSPCTL_WRITE_HCTR,&val);
   val=0;
   ioctl(fd,MDSPCTL_READ_HCTR,&val);
   success(val!=0);
   printf("Uploading program..."); fflush(stdout);
   success(write(fd,(char *)prog,sizeof(prog))==sizeof(prog));
   val=(val&(~HCTR_HRF1))|HCTR_HRF0|HCTR_HF0; /* Force start of program */
   ioctl(fd,MDSPCTL_WRITE_HCTR,&val);
   ioctl(fd,MDSPCTL_READ_HSTR,&val);
   if(val&HSTR_HRRQ) printf("Program running.\n");
  }
 printf("Select()ing DSP for read (10s)..."); fflush(stdout);
 FD_ZERO(&fds);
 FD_SET(fd,&fds);
 result=select(FD_SETSIZE,&fds,NULL,NULL,&tv);
 printf("returned after %d sec, %d usec, status=%d\n",tc.tv_sec-tv.tv_sec,
                                                      tc.tv_usec-tv.tv_usec,
                                                      result);
 printf("Select()ing DSP for write (10s)..."); fflush(stdout);
 tv=tc;
 FD_ZERO(&fds);
 FD_SET(fd,&fds);
 result=select(FD_SETSIZE,NULL,&fds,NULL,&tv);
 printf("returned after %d sec, %d usec, status=%d\n",tc.tv_sec-tv.tv_sec,
                                                      tc.tv_usec-tv.tv_usec,
                                                      result);
 printf("Benchmarking read()..."); fflush(stdout);
 gettimeofday(&tv,NULL);
 for(i=0,totalbytes=0;i<0x00ff;i++)
  {
   if((result=read(fd,buffer,sizeof(buffer)))!=sizeof(buffer))
    {
     fprintf(stderr,"Error in read, only %d of %d bytes read.\n",
             result,sizeof(buffer));
    }
   totalbytes+=result;
  }
 gettimeofday(&tv2,NULL);
 msecs=(tv2.tv_sec-tv.tv_sec)*1000+(tv2.tv_usec-tv.tv_usec)/1000;
 printf("Read %lu bytes in %d.%03d secs.\nThroughput was %d Bytes/sec.\n",
        (unsigned long)totalbytes,(int)(msecs/1000),(int)(msecs%1000),
        (int)(totalbytes/(msecs/1000)));
 printf("Benchmarking write()..."); fflush(stdout);
 gettimeofday(&tv,NULL);
 for(i=0,totalbytes=0;i<0x00ff;i++)
  {
   if((result=write(fd,buffer,sizeof(buffer)))!=sizeof(buffer))
    {
     fprintf(stderr,"Error in write, only %d of %d bytes written.\n",
             result,sizeof(buffer));
    }
   totalbytes+=result;
  }
 gettimeofday(&tv2,NULL);
 msecs=(tv2.tv_sec-tv.tv_sec)*1000+(tv2.tv_usec-tv.tv_usec)/1000;
 printf("Wrote %lu bytes in %d.%03d secs.\nThroughput was %d Bytes/sec.\n",
        (unsigned long)totalbytes,(int)(msecs/1000),(int)(msecs%1000),
        (int)(totalbytes/(msecs/1000)));
 close(fd);
 return 0;
}
